#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"

#include <ExceptionErrorHandler.h>

#include <qcanvas.h>
#include <qtimer.h>
#include <qpainter.h>
#include <qwidget.h>
#include <cmath>
#include "graph.h"
#include "OmSourceParameters.h" 
#include "OmProgressDialog.h"

/********************************************************************************
 *           
 *
 *******************************************************************************/

F90MODPROCPROTO(void, omsource_glue, getthecounts)(const float &radius);
F90MODPROCPROTO(void, omsource_glue, getsourcecounts)(const float &x, 
             const float &y, const float &radius, float &counts);
F90MODPROCPROTO(float, omsource_glue, getpsffraction)(const float &x, 
             const float &y, const float &radius);


F90MODPROCPROTO(void, omsource_glue, getthebackgroundcounts)(const float &x, 
             const float &y, const float &radius, float &r1, float &r2, float &counts);

extern F90MODVARDEF(PhotometryParameters, omsource_glue, myphotometryparameters);
 
 
Graph::Graph(QWidget *parent, OmSourceParameter *source, 
         QString xLabel, QString yLabel, QString title) :
  _parent(parent), _source(source), 
   _x(0), _y(0), _y1(0), _minX(0.0), _maxX(0.0),
  _minY(0.0), _maxY(0.0), _size(0), _ninX(10), _ninY(10),
   _xLabel(xLabel), _yLabel(yLabel), _title(title)
{

};


Graph::~Graph()
{
  if (_x)  delete [] _x;
  if (_y)  delete [] _y;
 

}


/************************************************************
 *
 ***********************************************************/
void Graph::getMinMax( )
{
    _minX = _x[0];
    _maxX = _minX;
    _minY = _y[0];
    _maxY = _minY;
    for (int i = 0; i < _size; i++)
    {
        if (_x[i] < _minX) _minX = _x[i];
        if (_x[i] > _maxX) _maxX = _x[i];

        if (_y[i] < _minY) _minY = _y[i];
        if (_y[i] > _maxY) _maxY = _y[i];


    }
    _minX = floor(_minX);
    _minY = floor(_minY);
    _maxX = ceil(_maxX);
    _maxY = ceil(_maxY);


};    
/************************************************************
 *
 ***********************************************************/
void Graph::drawGraph(QPainter *painter)
{
    
    QRect rect1 = _parent->rect();
    rect1.setLeft(0);
    rect1.setTop(0);
    rect1.setRight(1000);
    rect1.setBottom(800);

    painter->setViewport(rect1.left(), rect1.top(), 
                  rect1.right() - rect1.left(),
                  rect1.bottom() - rect1.top());
    int width = 100;
    rect1.setLeft( rect1.left() + width);
    rect1.setRight( rect1.right() - width );
 
    rect1.setTop( rect1.top() + width );
    width = 150; 
    rect1.setBottom( rect1.bottom() - width );


    QBrush brush( Qt::white, Qt::SolidPattern );          // white 12% fill
  
    painter->setBrush(brush);
                             
    painter->drawRect(rect1);
    QPen blackPen(Qt::black, 2);
    painter->setPen(blackPen);
    QPen bluePen(Qt::blue, 2);
    QPen redPen(Qt::red, 2);
  
    
    // Draw bottom axis
    double dwX = (rect1.right() - rect1.left()) / float(_ninX - 1);
    double dx = (_maxX - _minX) / float(_ninX - 1);
    int yPos = rect1.bottom();    

    for (int i = 0; i < _ninX; i++)
    {
      double xVal = _minX + double(i) * dx;
      int xPos = int((double)rect1.left() + double(i) * dwX);
      QString text;
      text.sprintf("%7.2f", xVal);
      QRect r = painter->fontMetrics().boundingRect(text );
  
      //      painter->drawText(r, Qt::AlignHCenter, text);
      //    painter->drawText(xPos, yPos, 100, 100, Qt::AlignHCenter, text);
      //   painter->drawText(xPos, yPos,  r.right() - r.left(), r.bottom() - r.top(), Qt::AlignRight, text);

      int width =  r.right() - r.left() + 5;
      int height =  r.bottom() - r.top() + 5;

      painter->drawText(xPos - width / 2, yPos + height + 10,  width, height, Qt::AlignCenter, text);
  
      painter->drawLine(xPos, yPos, xPos, yPos + 5);
      painter->drawLine(xPos, rect1.top(), xPos, rect1.top() - 5);


    }
    
    
    painter->drawLine(rect1.left(), yPos, rect1.right(), yPos);
    painter->drawLine(rect1.left(), rect1.top(), rect1.right(), rect1.top());
  
    
    // Draw Left axis
    _minY = 0.0;
    _maxY = 1.2;
    _ninY = 7;
    double dwY = (rect1.bottom() - rect1.top()) / float(_ninY - 1);
    double dy = (_maxY - _minY) / float(_ninY - 1);
    int xPos = rect1.left();    
    
  


    for (int i = 0; i < _ninY; i++)
    {
      double yVal = _minY + double(i) * dy;
      int yPos = int((double)rect1.bottom() - double(i) * dwY);
      QString text;
      text.sprintf("%4.2f", yVal);
      QRect r = painter->fontMetrics().boundingRect(text );
  
      //      painter->drawText(r, Qt::AlignRight, text);
      int width = r.right() - r.left() + 5;
      int height = r.bottom() - r.top() + 5;

      painter->drawText(xPos - width - 10, yPos - height / 2,  width, height, Qt::AlignRight, text);
  

     
      
      painter->drawLine(xPos, yPos, xPos - 5, yPos);
      painter->drawLine(rect1.right(), yPos, rect1.right() - 5, yPos);



    }
    


    painter->drawLine(xPos, rect1.bottom(), xPos, rect1.top());
    painter->drawLine(rect1.right(), rect1.bottom(), rect1.right(), rect1.top());


    dx = _maxX - _minX;
    dy = _maxY - _minY; 
    dwX = double (rect1.right() - rect1.left());

    dwY = double(rect1.bottom() - rect1.top());
    int xc1 = 0, yc1 = 0, yc3 = 0;
   
  

    // Plot the points

    for (int i = 0; i < _size; i++)
    {
      int xc = int(double(xPos) + dwX * (_x[i] - _minX) / dx);
      int yc = int(double(yPos) - dwY * (_y[i] - _minY) / dy);
    

      // The psf curve  
      int yc2 = int(double(yPos) - dwY * (_y1[i] - _minY) / dy);
    
    
      painter->setPen(redPen);

      painter->drawPoint(xc, yc);

      painter->setPen(bluePen);

      painter->drawPoint(xc, yc2);
       
      if (i > 0) 
      {
          painter->setPen(redPen);
          painter->drawLine(xc1, yc1, xc, yc);
          painter->setPen(bluePen);
          painter->drawLine(xc1, yc3, xc, yc2);

      }
      xc1 = xc;
      yc1 = yc;
      yc3 = yc2;


    }
    painter->setPen(blackPen);
    
    QRect r = painter->fontMetrics().boundingRect(_xLabel);
  
   
    width = r.right() - r.left() + 30;
    int height = r.bottom() - r.top() + 30;
    QFont font1("Times", 17, QFont::Bold);
    painter->setFont(font1);
    painter->drawText(rect1.left() + (rect1.right() - rect1.left()) / 2, rect1.bottom() + 50,  
            width, height, Qt::AlignCenter, _xLabel);
  
    QFont font2("Times", 20, QFont::Bold);
    painter->setFont(font2);
 
 
    r = painter->fontMetrics().boundingRect(_title);
  
   
    width = r.right() - r.left() + 30;
    height = r.bottom() - r.top() + 30;

    painter->drawText((rect1.right() - rect1.left() - width) / 2, rect1.top() - 50,  
            width, height, Qt::AlignCenter, _title);
  

    QWMatrix m;
    m.rotate(-90 );
    painter->setWorldMatrix( m, TRUE); 

		//	    painter->rotate(-90);


	    //    painter->rotate(90.0);

    r = painter->fontMetrics().boundingRect(_yLabel);
  
   
    width = r.right() - r.left() + 30;
    height = r.bottom() - r.top() + 30;
    painter->setFont(font1);
 
    
    painter->drawText(-(rect1.bottom() - width) / 2, (rect1.left() - width) / 2,  _yLabel);
    
  
};
/*****************************************************************
 *
 ****************************************************************/
void OmSourceGraph::loadData(int size)
{




    _size = size;
    if (_x) delete [] _x;
    if (_y) delete [] _y;
    if (_y1) delete [] _y1;
    


    for (int i = 0; i < 10; i++)
    QApplication::beep();


   
    _x = new double[size];
    _y = new double[size];
    _y1 = new double[size];
   


  /*****************************************************************************
     * Need the cal aperture radius (binned) for doing photometry
     ***************************************************************************/
    PhotometryParameters photParameters = F90MODVAR(omsource_glue, myphotometryparameters);
  
    float apertureRadius =  photParameters.apertureRadius;

    float binning =  float(photParameters.binX);

    float backgroundCounts;
    float r1 = 3.0;



    float r2 = apertureRadius + 2.0;
    float r3 = r2 + 4.0;
   
    F90MODPROC(omsource_glue, getthebackgroundcounts)(_source->xcoord, 
                  _source->ycoord, r1, r2, r3, backgroundCounts);
   
    float dr = (r2 - 2.0 - r1) / float(size - 1);
    OmProgressDialog  *progress = new OmProgressDialog("Computing growth-curve- please wait", size);    

    for (int i = 0; i < size; i++)
    {
      float radius = r1 + float(i) * dr;
      float counts;
      F90MODPROC(omsource_glue, getsourcecounts)(_source->xcoord, 
                  _source->ycoord, radius, counts);
      counts = counts - backgroundCounts * radius * radius / (r1 * r1);
      _x[i] = radius;      
      _y[i] = counts;    
      cout << "radius=" << radius << " counts=" << counts << endl;
    
      _y1[i] = F90MODPROC(omsource_glue, getpsffraction)(_source->xcoord, 
                  _source->ycoord, radius * binning);
  


 
      progress->setProgress(i);
    }
    
    // Normalize
    for (int i = 1; i < size; i++)
    {
        _y[i] /= _y[size - 1]; 
        _y1[i] /= _y1[size - 1]; 
   
    }
    _y[size - 1] = 1.0;
   _y1[size - 1] = 1.0;

    delete progress;   
    
    getMinMax();   
}     
     









